@@ -114,8 +114,8 @@ ConfigPath::ConfigPath(wxWindow* parent)
 			std::vector<std::string> toks;
 			std::string s_dir(dir.path.ToUTF8());
 			TokenizeMail(s_dir, toks, "|");
-			dir.path = toks[0];
-			dir.name = toks[1];
+			dir.path = wxString::FromUTF8(toks[0].c_str());
+			dir.name = wxString::FromUTF8(toks[1].c_str());
 			if (toks.size()>2)
 			{
 				dir.group = (std::max)(0, atoi(toks[2].c_str()));
@@ -214,7 +214,11 @@ void ConfigPath::OnClickNew(wxCommandEvent &evt)
 		ad.server_default = 0;
 		dirs_client.push_back(ad);
 
-		default_dirs_use |= c_use_value_client;
+		if (dirs_group.empty() && dirs_home.empty())
+			default_dirs_use = c_use_value_client;
+		else
+			default_dirs_use |= c_use_value_client;
+
 		switchBitmapLabel();
 		renderListBoxContent();
 
@@ -465,4 +469,5 @@ void ConfigPath::OnClickSourceSwitch(wxCommandEvent & event)
 
 	renderListBoxContent();
 	switchBitmapLabel();
+	mod = true;
 }
