@@ -500,7 +500,8 @@ std::vector<SBackupDir> Connector::getSharedPaths(void)
 				wxString::FromUTF8(dir["name"].asCString()),
 				dir["id"].asInt(),
 				dir["group"].asInt(),
-				wxString::FromUTF8(dir["flags"].asCString())
+				wxString::FromUTF8(dir["flags"].asCString()),
+				dir["server_default"].asInt()
 			};
 
 			ret.push_back(rdir);
@@ -516,9 +517,13 @@ std::vector<SBackupDir> Connector::getSharedPaths(void)
 bool Connector::saveSharedPaths(const std::vector<SBackupDir> &res)
 {
 	std::string args;
+	size_t idx = 0;
 	for(size_t i=0;i<res.size();++i)
 	{
-		if(i!=0)
+		if (res[i].server_default)
+			continue;
+
+		if(!args.empty())
 			args+="&";
 
 		std::string path = EscapeParamString(res[i].path.ToUTF8().data());
@@ -530,9 +535,10 @@ bool Connector::saveSharedPaths(const std::vector<SBackupDir> &res)
 			name += "/" + res[i].flags;
 		}
 
-		args+="dir_"+nconvert(i)+"="+path;
-		args+="&dir_"+nconvert(i)+"_name="+name;
-		args+="&dir_"+nconvert(i)+"_group="+nconvert(res[i].group);
+		args+="dir_"+nconvert(idx)+"="+path;
+		args+="&dir_"+nconvert(idx)+"_name="+name;
+		args+="&dir_"+nconvert(idx)+"_group="+nconvert(res[i].group);
+		idx += 1;
 	}
 
 	std::string d=getResponse("SAVE BACKUP DIRS", args, true);
