@@ -431,11 +431,11 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent),
 		&& internet_server_url.find(L"wss://") != 0
 		&& internet_server_url.find(L"ws://") != 0)
 	{
-		if (watoi(internet_server_port) == 55415)
+		if (watoi(internet_server_port) == 55415 && !internet_server_url.empty())
 		{
 			internet_server_url = "urbackup://" + internet_server_url;
 		}
-		else
+		else if(!internet_server_url.empty() && !internet_server_port.empty())
 		{
 			internet_server_url = "urbackup://" + internet_server_url + ":" + internet_server_port;
 		}
@@ -799,12 +799,12 @@ void Settings::OnOkClick( wxCommandEvent& event )
 	{
 		std::wstring old_val;
 		std::wstring old_use;
-		if (it->second.use != c_use_group
-			&& it->second.use!=c_use_value
-			&& ((!settings->getValue(it->first+L".client", &old_val) &&
-				old_val!=it->second.value_client) ||
-				( !settings->getValue(it->first + L".use", &old_use) &&
-				old_use!=convert(it->second.use) ) )  )
+		if ((it->second.use != c_use_group &&
+			 it->second.use!=c_use_value &&
+			  (!settings->getValue(it->first+L".client", &old_val) ||
+				old_val!=it->second.value_client) ) ||
+				( !settings->getValue(it->first + L".use", &old_use) ||
+				old_use!=convert(it->second.use) ) )  
 		{
 			std::string key(wxString(it->first).ToUTF8());
 			s_data += key + ".client=" + wxString(it->second.value_client).ToUTF8() + "\n";
@@ -1059,9 +1059,15 @@ std::wstring Settings::transformValFromUI(const std::wstring & key, const std::w
 void Settings::setSettingsSwitch(const std::wstring & key, wxBitmapButton* btn, wxWindow* ctrl)
 {
 	std::wstring use_str;
-	settings->getValue(key + L".use", &use_str);
-
-	int use = watoi(use_str);;
+	int use;
+	if (!settings->getValue(key + L".use", &use_str))
+	{
+		use = c_use_group;
+	}
+	else
+	{
+		use = watoi(use_str);;
+	}
 
 	if (use == c_use_value_client)
 	{
