@@ -17,11 +17,20 @@
 **************************************************************************/
 
 #include <wx/wx.h>
-
+#include <map>
 #include "Connector.h"
 #include "gui/GUI.h"
 #include "FileSettingsReader.h"
 
+namespace
+{
+	const int c_use_group = 1;
+	const int c_use_value = 2;
+	const char* c_use_value_str = "2";
+	const int c_use_value_client = 4;
+	const char* c_use_value_client_str = "4";
+}
+
 
 class Settings : public GUISettings
 {
@@ -32,9 +41,32 @@ public:
 	virtual void OnOkClick( wxCommandEvent& event );
 	virtual void OnAbortClick( wxCommandEvent& event );
 	virtual void OnDisableImageBackups( wxCommandEvent& event );
-
-	static std::string mergeNewSettings(CFileSettingsReader *settings, const std::map<std::string, std::string>& n_vals);
+	virtual void OnBitmapBtnClick(wxCommandEvent& event);
+	virtual void OnCtlChange(wxCommandEvent& event);
 
 private:
+
+	std::wstring transformValToUI(const std::wstring& key, const std::wstring& val);
+	std::wstring transformValFromUI(const std::wstring& key, const std::wstring& val);
+
+	void setSettingsSwitch(const std::wstring& key, wxBitmapButton* btn, wxWindow* ctrl);
+
+	std::map<wxWindowID, std::wstring> button_ids;
+	std::map<wxWindowID, std::wstring> ctrl_ids;
+
+	struct SSetting
+	{
+		wxBitmapButton* btn;
+		wxWindow* ctrl;
+		int use;
+		std::wstring value_group;
+		std::wstring value_home;
+		std::wstring value_client;
+	};
+
+	std::map<std::wstring, SSetting> settings_info;
+
 	CFileSettingsReader *settings;
+
+	bool init_complete;
 };
\ No newline at end of file
