@@ -22,6 +22,7 @@
 #include <sstream> 
 #include <map>
 #include "utf8/utf8.h"
+#include "stringtools.h"
 
 #ifndef _WIN32
 #include <memory.h>
@@ -666,7 +667,7 @@ std::string replaceonce(std::string tor, std::string tin, std::string data)
 	between the tokens) - strings, which are declared by two \" are not
 	tokenized = treated as a single token
 */
-void	Tokenize(std::string& str, std::vector<std::string> &tokens, std::string seps)
+void	Tokenize(const std::string& str, std::vector<std::string> &tokens, std::string seps)
 {
 	// one-space line for storing blank lines in the file
 	std::string blankLine = " ";
@@ -722,7 +723,7 @@ void	Tokenize(std::string& str, std::vector<std::string> &tokens, std::string se
 		if(tokens[i]=="") tokens.erase(tokens.begin()+(i--));
 }
 
-void	Tokenize(std::wstring& str, std::vector<std::wstring> &tokens, std::wstring seps)
+void	Tokenize(const std::wstring& str, std::vector<std::wstring> &tokens, std::wstring seps)
 {
 	// one-space line for storing blank lines in the file
 	std::wstring blankLine = L" ";
@@ -778,7 +779,7 @@ void	Tokenize(std::wstring& str, std::vector<std::wstring> &tokens, std::wstring
 		if(tokens[i].empty()) tokens.erase(tokens.begin()+(i--));
 }
 
-void TokenizeMail(std::string& str, std::vector<std::string> &tokens, std::string seps)
+void TokenizeMail(const std::string& str, std::vector<std::string> &tokens, std::string seps)
 {
 	// one-space line for storing blank lines in the file
 	std::string blankLine = "";
@@ -1037,8 +1038,6 @@ std::wstring UnescapeSQLString(std::wstring pStr)
 	return pStr;
 }
 
-string htmldecode(string str, bool html, char xc='%');
-
 void ParseParamStrHttp(const std::string &pStr, std::map<std::string, std::string> *pMap)
 {
 	std::string key;
@@ -1497,4 +1496,47 @@ std::string EscapeParamString(const std::string &pStr)
 		}
 	}
 	return ret;
+}
+
+std::string EscapePathParamString(const std::string & pStr)
+{
+	std::string ret;
+	ret.reserve(pStr.size());
+	for (size_t i = 0; i<pStr.size(); ++i)
+	{
+		switch (pStr[i])
+		{
+		case '%': ret += "%25"; break;
+		case '/': ret += "%2F"; break;
+		case '|': ret += "%7C"; break;
+		case ';': ret += "%3B"; break;
+		default: ret += pStr[i]; break;
+		}
+	}
+	return ret;
+}
+
+std::string UnescapeParamString(const std::string & str)
+{
+	char xc = '%';
+	std::string ret;
+	ret.reserve(str.size());
+	for (size_t i = 0; i<str.size(); i++)
+	{
+		if (str[i] == xc && i + 2<str.size())
+		{
+			std::string data; data.push_back(str[i + 1]); data.push_back(str[i + 2]);
+			unsigned char ch = (unsigned char)hexToULong(data);
+			if (ch != 0)
+			{
+				ret += ch;
+			}
+			i += 2;
+		}
+		else
+		{
+			ret += str[i];
+		}
+	}
+	return ret;
 }
\ No newline at end of file
