@@ -14,6 +14,7 @@
 #	include <netdb.h>
 #	include <unistd.h>
 #	include <fcntl.h>
+#	include <errno.h>
 #	define SOCKET_ERROR -1
 #	define closesocket close
 #	define SOCKET int
@@ -27,7 +28,7 @@
 #include "config.h"
 #endif
 #ifdef HAVE_ACCEPT4
-#define ACCEPT_CLOEXEC(sockfd, addr, addrlen) accept4(sockfd, addr, addrlen, SOCK_CLOEXEC)
+#define ACCEPT_CLOEXEC(sockfd, addr, addrlen) accept4_fb(sockfd, addr, addrlen, SOCK_CLOEXEC)
 #else
 #define EMULATE_ACCEPT_CLOEXEC
 #endif
@@ -50,4 +51,20 @@ namespace {
 	}
 }
 #endif //ACCEPT_CLOEXEC_DEFINED
-#endif //EMULATE_ACCEPT_CLOEXEC
\ No newline at end of file
+#endif //EMULATE_ACCEPT_CLOEXEC
+
+#if !defined(EMULATE_ACCEPT_CLOEXEC) && defined(SOCK_CLOEXEC) && defined(HAVE_ACCEPT4) && !defined(ACCEPT_FB_DEFINED)
+#define ACCEPT_FB_DEFINED
+namespace {
+	int accept4_fb(int sockfd, struct sockaddr *addr, socklen_t *addrlen, int flags)
+	{
+		int rc = accept4(sockfd, addr, addrlen, flags);
+		if (rc == -1 && errno == EINVAL)
+		{
+			rc = accept(sockfd, addr, addrlen);
+			if (rc) fcntl(rc, F_SETFD, fcntl(rc, F_GETFD, 0) | FD_CLOEXEC);
+		}
+		return rc;
+	}
+}
+#endif
\ No newline at end of file
