@@ -37,6 +37,8 @@
 #include "win_all_volumes.h"
 #include "DirectoryWatcherThread.h"
 #include "win_network_cost.h"
+#include <mutex>
+#include <thread>
 #else
 #include "lin_ver.h"
 #include "lin_sysvol.h"
@@ -1175,6 +1177,59 @@ void ClientConnector::CMD_UPDATE_SETTINGS(const std::string &cmd)
 	lasttime=Server->getTimeMS();
 }
 
+#ifdef _WIN32
+
+namespace
+{
+	std::mutex prevent_sleep_mutex;
+	int64 last_prevent_sleep_time = 0;
+	std::thread prevent_sleep_thread;
+
+	void prevent_thread_func()
+	{
+		SetThreadExecutionState(ES_SYSTEM_REQUIRED | ES_CONTINUOUS);
+
+		std::unique_lock<std::mutex> lock(prevent_sleep_mutex);
+
+		while (Server->getTimeMS() - last_prevent_sleep_time < 2 * 60 * 1000)
+		{
+			lock.unlock();
+			Server->wait(10000);
+			lock.lock();
+		}
+
+		last_prevent_sleep_time = 0;
+		SetThreadExecutionState(ES_CONTINUOUS);
+	}
+
+	void preventSleep()
+	{
+		OSVERSIONINFO verinfo = {};
+		verinfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+		//Check for Win11
+		if (GetVersionExW(&verinfo) &&
+			(verinfo.dwMajorVersion > 10 ||
+				(verinfo.dwMajorVersion == 10 && verinfo.dwMinorVersion > 0) ||
+				(verinfo.dwMajorVersion == 10 && verinfo.dwMinorVersion == 0 &&
+					verinfo.dwBuildNumber >= 22000)))
+		{
+			std::lock_guard<std::mutex> lock(prevent_sleep_mutex);
+
+			if (last_prevent_sleep_time == 0)
+			{
+				prevent_sleep_thread = std::thread(prevent_thread_func);
+			}
+
+			last_prevent_sleep_time = Server->getTimeMS();
+			return;
+		}
+
+		SetThreadExecutionState(ES_SYSTEM_REQUIRED);
+	}
+}
+
+#endif //_WIN32
+
 void ClientConnector::CMD_PING_RUNNING(const std::string &cmd)
 {
 	tcpstack.Send(pipe, "OK");
@@ -1212,7 +1267,7 @@ void ClientConnector::CMD_PING_RUNNING(const std::string &cmd)
 	proc->last_pingtime = Server->getTimeMS();	
 
 #ifdef _WIN32
-	SetThreadExecutionState(ES_SYSTEM_REQUIRED);
+	preventSleep();
 #endif
 }
 
@@ -1376,15 +1431,53 @@ void ClientConnector::CMD_TOCHANNEL_START_INCR_IMAGEBACKUP(const std::string &cm
 
 void ClientConnector::CMD_TOCHANNEL_UPDATE_SETTINGS(const std::string &cmd)
 {
+	std::string s_settings = cmd.substr(16);
+	unescapeMessage(s_settings);
+
 	if(last_capa & DONT_SHOW_SETTINGS)
 	{
-		tcpstack.Send(pipe, "FAILED");
-		return;
+		bool change_allowed = false;
+		if (!(last_capa & DONT_ALLOW_CONFIG_PATHS) ||
+			!(last_capa & DONT_ALLOW_COMPONENT_CONFIG) )
+		{
+			std::auto_ptr<ISettingsReader> new_settings(Server->createMemorySettingsReader(s_settings));
+			bool default_dirs_use = false;
+			std::vector<std::string> keys = new_settings->getKeys();
+			unsigned int settings_types = 0;
+			for (size_t i = 0; i < keys.size(); ++i)
+			{
+				const std::string& key = keys[i];
+				if (key == "set_client_settings")
+					continue;
+				if (key == "default_dirs" || key.find("default_dirs.") == 0)
+					settings_types |= 1;
+				else if (key == "vss_select_components" || key.find("vss_select_components.") == 0)
+					settings_types |= 2;
+				else
+				{
+					settings_types |= 4;
+					break;
+				}
+			}
+
+			if (settings_types == 1 && 
+				!(last_capa & DONT_ALLOW_CONFIG_PATHS) )
+				change_allowed = true;
+			else if (settings_types == 2 &&
+				!(last_capa & DONT_ALLOW_COMPONENT_CONFIG))
+				change_allowed = true;
+		}
+
+		if (!change_allowed)
+		{
+			Server->Log("Settings change not allowed: "+ s_settings, LL_INFO);
+			tcpstack.Send(pipe, "FAILED");
+			return;
+		}
 	}
 
-	std::string s_settings=cmd.substr(16);
+	
 	lasttime=Server->getTimeMS();
-	unescapeMessage(s_settings);
 	replaceSettings( s_settings );
 
 	IScopedLock lock(backup_mutex);
