@@ -1303,13 +1303,12 @@ std::string ImageThread::hdatFn(std::string volume)
 
 IFsFile* ImageThread::openHdatF(std::string volume, bool share)
 {
+#ifdef _WIN32
 	if (!IndexThread::normalizeVolume(volume))
 	{
 		return NULL;
 	}
 
-#ifdef _WIN32
-
 	DWORD share_mode = FILE_SHARE_READ;
 
 	if (share)
@@ -1327,7 +1326,7 @@ IFsFile* ImageThread::openHdatF(std::string volume, bool share)
 
 	return Server->openFileFromHandle(hfile, volume + os_file_sep() + "System Volume Information\\urbhdat_img.dat");
 #else
-	return NULL;
+	return Server->openFile("urbackup/hdat_img_"+conv_filename(volume)+".dat", MODE_RW_CREATE);
 #endif
 }
 
