@@ -44,6 +44,12 @@
 #include <fstream>
 #include <sstream>
 
+#ifdef __APPLE__
+#include <iostream>
+#include <sys/xattr.h>
+#include <sys/types.h>
+#endif
+
 const int c_group_vss_components = -1;
 const int c_group_default = 0;
 const int c_group_continuous = 1;
@@ -60,6 +66,7 @@ const unsigned int flag_with_orig_path = 16;
 const unsigned int flag_with_sequence = 32;
 const unsigned int flag_with_proper_symlinks = 64;
 
+
 const uint64 change_indicator_symlink_bit = 0x4000000000000000ULL;
 const uint64 change_indicator_special_bit = 0x2000000000000000ULL;
 const uint64 change_indicator_all_bits = change_indicator_symlink_bit | change_indicator_special_bit;
@@ -80,9 +87,18 @@ private:
 	volatile static bool pause;
 };
 
+enum CbtType
+{
+	CbtType_None,
+	CbtType_Datto,
+	CbtType_Era
+};
+
 struct SCRef
 {
-	SCRef(void): ok(false), dontincrement(false), cbt(false), for_imagebackup(false), with_writers(false) {
+	SCRef(void): ok(false), dontincrement(false), cleanup(false), cbt(false),
+		for_imagebackup(false), with_writers(false),
+		cbt_type(CbtType_None) {
 #ifdef _WIN32
 		backupcom = NULL;
 #endif
@@ -99,11 +115,20 @@ struct SCRef
 	int save_id;
 	bool ok;
 	bool dontincrement;
+	bool cleanup;
+	size_t cleanup_gen;
 	std::vector<std::string> starttokens;
+	std::vector<std::string> sharenames;
 	std::string clientsubname;
 	bool cbt;
 	bool for_imagebackup;
 	bool with_writers;
+	std::string cbt_file;
+	CbtType cbt_type;
+
+#ifndef _WIN32
+	std::vector<IFsFile::os_file_handle> flock_fds;
+#endif
 };
 
 struct SCDirs
@@ -181,6 +206,7 @@ struct SBackupScript
 	std::string outputname;
 	int64 size;
 	std::string orig_path;
+	int64 lastmod;
 
 	bool operator<(const SBackupScript &other) const
 	{
@@ -310,6 +336,11 @@ struct SQueueRef
 	}
 };
 
+#ifdef _WIN32
+struct _URBCT_BITMAP_DATA;
+typedef struct _URBCT_BITMAP_DATA* PURBCT_BITMAP_DATA;
+#endif
+
 class IndexThread : public IThread, public IFileServ::IReadErrorCallback, public IDeregisterFileSrvScriptFn
 {
 public:
@@ -351,6 +382,7 @@ public:
 	
 	static bool backgroundBackupsEnabled(const std::string& clientsubname);
 
+	static std::vector<std::string> buildExcludeList(const std::string& val);
 	static std::vector<std::string> parseExcludePatterns(const std::string& val);
 	static std::vector<SIndexInclude> parseIncludePatterns(const std::string& val);
 
@@ -383,6 +415,10 @@ public:
 	static bool unrefResult(unsigned int id);
 
 	static void removeResult(unsigned int id);
+
+#ifndef _WIN32
+	static std::string get_snapshot_script_location(const std::string& scriptname, const std::string& index_clientsubname);
+#endif
 	
 private:
 	static void addResult(unsigned int id, const std::string& res);
@@ -412,9 +448,9 @@ private:
 
 	enum IndexErrorInfo
 	{
-		IndexErrorInfo_Ok,
-		IndexErrorInfo_Error,
-		IndexErrorInfo_NoBackupPaths
+		IndexErrorInfo_Ok = 0,
+		IndexErrorInfo_Error = 1,
+		IndexErrorInfo_NoBackupPaths = 2
 	};
 
 	IndexErrorInfo indexDirs(bool full_backup, bool simultaneous_other);
@@ -461,9 +497,9 @@ private:
 	void removeUnconfirmedVssDirs();
 	std::string expandPath(BSTR pathStr);
 	void removeBackupcomReferences(IVssBackupComponents *backupcom);
+	bool addFileToCbt(const std::string& fpath, const DWORD& blocksize, const PURBCT_BITMAP_DATA& bitmap_data);
 #else
 	bool start_shadowcopy_lin( SCDirs * dir, std::string &wpath, bool for_imagebackup, bool * &onlyref, bool* not_configured);
-	std::string get_snapshot_script_location(const std::string& name);
 	bool get_volumes_mounted_locally();
 	bool getVssSettings() { return true; }
 #endif
@@ -478,9 +514,9 @@ private:
 
 	SCDirs* getSCDir(const std::string& path, const std::string& clientsubname, bool for_imagebackup);
 
-	int execute_hook(std::string script_name, bool incr, std::string server_token, int* index_group);
+	int execute_hook(std::string script_name, bool incr, std::string server_token, int* index_group, int* error_info=NULL);
 	int execute_prebackup_hook(bool incr, std::string server_token, int index_group);
-	int execute_postindex_hook(bool incr, std::string server_token, int index_group);
+	int execute_postindex_hook(bool incr, std::string server_token, int index_group, IndexErrorInfo error_info);
 	std::string execute_script(const std::string& cmd, const std::string& args);
 
 	int execute_preimagebackup_hook(bool incr, std::string server_token);
@@ -519,6 +555,17 @@ private:
 	static void addFileExceptions(std::vector<std::string>& exclude_dirs);
 
 	static void addHardExcludes(std::vector<std::string>& exclude_dirs);
+    
+#ifdef __APPLE__
+    static std::vector<std::string> macos_exclusions;
+    static std::vector<std::string> macos_overrides;
+    
+    static void addMacOSExcludes(std::vector<std::string>& exclude_dirs);
+    static void logMacOSExclude(const std::string path);
+    static std::string returnExcludeDirsMatch(std::vector<std::string> exclude_dirs, std::string path);
+    static bool isExcludedByXattr(const std::string path);
+    static void logIsExcludedByXattr(const std::string path);
+#endif
 
 	void handleHardLinks(const std::string& bpath, const std::string& vsspath, const std::string& normalized_volume);
 
@@ -568,7 +615,15 @@ private:
 
 	bool prepareCbt(std::string volume);
 
-	bool finishCbt(std::string volume, int shadow_id, std::string snap_volume, bool for_image_backup);
+	bool finishCbt(std::string volume, int shadow_id, std::string snap_volume, bool for_image_backup, std::string cbt_file, CbtType cbt_type);
+
+#ifndef _WIN32
+	bool finishCbtDatto(IFile* volfile, IFsFile* hdat_file, IFsFile* hdat_img, std::string volume, int shadow_id, std::string snap_volume, bool for_image_backup, std::string cbt_file);
+#endif
+
+	bool finishCbtEra(IFsFile* hdat_file, IFsFile* hdat_img, std::string volume, int shadow_id, std::string snap_volume, bool for_image_backup, std::string cbt_file, int64& hdat_file_era, int64& hdat_img_era);
+
+	bool finishCbtEra2(IFsFile* hdat, int64 hdat_era);
 
 	bool disableCbt(std::string volume);
 
@@ -604,6 +659,10 @@ private:
 
 	bool isAllSpecialDir(const SBackupDir& bdir);
 
+	bool punchHoleOrZero(IFile* f, int64 pos, const char* zero_buf, char* zero_read_buf, size_t zero_size);
+
+	void run_sc_refs_cleanup();
+
 	SVolumesCache* volumes_cache;
 
 	std::auto_ptr<ScopedBackgroundPrio> background_prio;
@@ -639,6 +698,8 @@ private:
 	std::map<SCDirServerKey, std::map<std::string, SCDirs*> > scdirs;
 	std::vector<SCRef*> sc_refs;
 
+	bool sc_refs_cleanup;
+
 	int index_c_db;
 	int index_c_fs;
 	int index_c_db_update;
@@ -841,6 +902,8 @@ private:
 	static unsigned int next_result_id;
 	static IMutex* result_mutex;
 
+	std::vector<IFsFile::os_file_handle> flock_fds_perm;
+
 #ifdef _WIN32
 	struct SComponent
 	{
