@@ -23,6 +23,11 @@
 #define DLLEXPORT extern "C"
 #endif
 
+#ifdef __linux__
+#include <linux/fs.h>
+#include <sys/ioctl.h>
+#endif
+
 #include <vector>
 
 #ifndef STATIC_PLUGIN
@@ -110,6 +115,7 @@ void do_restore(void);
 void restore_wizard(void);
 void upgrade(void);
 bool upgrade_client(void);
+void parse_devnum_test();
 
 std::string lang="en";
 std::string time_format_str_de="%d.%m.%Y %H:%M";
@@ -128,10 +134,114 @@ const std::string new_file="urbackup/new.txt";
 THREADPOOL_TICKET indexthread_ticket;
 THREADPOOL_TICKET internetclient_ticket;
 
+namespace
+{
+	int64 roundUp(int64 numToRound, int64 multiple)
+	{
+		return ((numToRound + multiple - 1) / multiple) * multiple;
+	}
+
+	int64 roundDown(int64 numToRound, int64 multiple)
+	{
+		return ((numToRound / multiple) * multiple);
+	}
+
+	const int64 dm_block_size = 1 * 1024 * 1024;
+
+	void print_ext(const IFsFile::SFileExtent& ext, const std::string& file_dm_block_dev, int64& lin_off)
+	{
+		int64 start = roundUp(ext.volume_offset, dm_block_size);
+		int64 end = roundDown(ext.volume_offset + ext.size, dm_block_size);
+
+		if (end - start >= dm_block_size)
+		{
+			int64 bcount = (end - start) / 512;
+			std::cout << lin_off << " " << bcount << " linear " << file_dm_block_dev << " " << (start / 512) << std::endl;
+			lin_off += bcount;
+		}
+	}
+
+	void do_print_dm_file_extents(const std::string& fn)
+	{
+		std::auto_ptr<IFsFile> f(Server->openFile(fn, MODE_READ));
+		if (f.get() == NULL)
+		{
+			std::cerr << "Error opening file " << fn << " " << os_last_error_str() << std::endl;
+			exit(1);
+		}
+
+#ifdef FS_IOC_FSSETXATTR
+		fsxattr attr = {};
+#ifdef FS_XFLAG_IMMUTABLE
+		attr.fsx_xflags |= FS_XFLAG_IMMUTABLE;
+#endif
+		ioctl(f->getOsHandle(), FS_IOC_FSSETXATTR, &attr);
+#ifdef FS_XFLAG_NODUMP
+		attr.fsx_xflags |= FS_XFLAG_NODUMP;
+#endif
+		ioctl(f->getOsHandle(), FS_IOC_FSSETXATTR, &attr);
+#ifdef FS_XFLAG_NODEFRAG
+		attr.fsx_xflags |= FS_XFLAG_NODEFRAG;
+#endif
+		ioctl(f->getOsHandle(), FS_IOC_FSSETXATTR, &attr);
+#endif
+
+		std::string file_dm_block_dev = Server->getServerParameter("file-dm-block-dev");
+
+
+		int64 lin_off = 0;
+		int64 pos = 0;
+		bool more_data = true;
+		
+		IFsFile::SFileExtent last_ext;
+		while (more_data)
+		{
+			std::vector<IFsFile::SFileExtent> exts = f->getFileExtents(pos, 0, more_data);
+
+			for (size_t i = 0; i < exts.size(); ++i)
+			{
+				if (last_ext.offset == -1)
+				{
+					last_ext = exts[i];
+				}
+				else if (last_ext.offset + last_ext.size != exts[i].offset
+					|| last_ext.volume_offset + last_ext.size != exts[i].volume_offset)
+				{
+					print_ext(last_ext, file_dm_block_dev, lin_off);
+					last_ext = exts[i];
+				}
+				else
+				{
+					last_ext.size += exts[i].size;
+				}
+
+				pos = (std::max)(exts[i].offset + exts[i].size, pos);
+			}
+		}
+
+		if (last_ext.offset != -1)
+		{
+			print_ext(last_ext, file_dm_block_dev, lin_off);
+		}
+
+		if ((lin_off * 512) < (f->Size() * 3) / 4)
+		{
+			std::cerr << "ERROR: Only " << PrettyPrintBytes(lin_off * 512) << " of " << PrettyPrintBytes(f->Size()) << " was usable" << std::endl;
+			exit(2);
+		}
+
+		exit(0);
+	}
+}
+
 
 DLLEXPORT void LoadActions(IServer* pServer)
 {
 	Server=pServer;
+
+#ifdef _DEBUG
+	parse_devnum_test();
+#endif
 	
 	std::string rmtest=Server->getServerParameter("rmtest");
 	if(!rmtest.empty())
@@ -155,6 +265,13 @@ DLLEXPORT void LoadActions(IServer* pServer)
 		return;
 	}
 
+	std::string print_dm_file_extents = Server->getServerParameter("print-dm-file-extents");
+	if (!print_dm_file_extents.empty())
+	{
+		do_print_dm_file_extents(print_dm_file_extents);
+		return;
+	}
+
 #ifdef _WIN32
 	char t_lang[20];
 	GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT,LOCALE_SISO639LANGNAME ,t_lang,sizeof(t_lang));
@@ -385,6 +502,19 @@ DLLEXPORT void LoadActions(IServer* pServer)
 	bool do_leak_check=(Server->getServerParameter("leak_check")=="true");
 
 	ClientConnector::init_mutex();
+
+	filesrv_pluginid = Server->StartPlugin("fileserv", params);
+
+	IndexThread* it = new IndexThread();
+	if (!do_leak_check)
+	{
+		Server->createThread(it, "file indexing");
+	}
+	else
+	{
+		indexthread_ticket = Server->getThreadPool()->execute(it, "file indexing");
+	}
+
 	unsigned short urbackup_serviceport = default_urbackup_serviceport;
 	if(!Server->getServerParameter("urbackup_serviceport").empty())
 	{
@@ -399,18 +529,6 @@ DLLEXPORT void LoadActions(IServer* pServer)
 
 	Server->StartCustomStreamService(new ClientService(), "urbackupserver", urbackup_serviceport, -1, serviceport_bind_target);
 
-	filesrv_pluginid=Server->StartPlugin("fileserv", params);
-
-	IndexThread *it=new IndexThread();
-	if(!do_leak_check)
-	{
-		Server->createThread(it, "file indexing");
-	}
-	else
-	{
-		indexthread_ticket=Server->getThreadPool()->execute(it, "file indexing");
-	}
-
 	internetclient_ticket=InternetClient::start(do_leak_check);
 
 #ifdef _WIN32
@@ -636,6 +754,11 @@ void update_client26_27(IDatabase* db)
 	db->Write("ALTER TABLE files ADD generation INTEGER DEFAULT 0");
 }
 
+void update_client27_28(IDatabase* db)
+{
+	ClientConnector::updateDefaultDirsSetting(db, true, 0, false);
+}
+
 bool upgrade_client(void)
 {
 	IDatabase *db=Server->getDatabase(Server->getThreadID(), URBACKUPDB_CLIENT);
@@ -647,7 +770,7 @@ bool upgrade_client(void)
 		return false;
 	int ver=watoi(res_v[0]["tvalue"]);
 	int old_v;
-	int max_v = 27;
+	int max_v = 28;
 
 	if (ver > max_v)
 	{
@@ -775,6 +898,10 @@ bool upgrade_client(void)
 				update_client26_27(db);
 				++ver;
 				break;
+			case 27:
+				update_client27_28(db);
+				++ver;
+				break;
 			default:
 				break;
 		}
